//+------------------------------------------------------------------+
//|                                              9Squared SMAC-D.mq4 |
//|                                   Copyright  2007, Steve Bowley |
//|                                        http://www.9squaredfx.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Steve Bowley"
#property link      "http://www.9squaredfx.com"

#property indicator_separate_window
#property indicator_buffers    2
#property indicator_color1     RoyalBlue
#property indicator_color2     Snow
#property indicator_levelcolor Gold

//
//
//
//
//

extern int    FastEMA         = 18;
extern int    SlowEMA         = 27;
extern int    SignalSMA       = 9;
extern string note            = "turn on Alert = true; turn off = false";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";
extern bool   arrowsVisible   = true;
extern string arrowsUniqueID  = "macd Arrows1";
extern color  arrowsUpColor   = DeepSkyBlue;
extern color  arrowsDnColor   = PaleVioletRed;
extern int    arrowsUpCode    = 241;
extern int    arrowsDnCode    = 242;
extern int    arrowsSize      = 1;

//
//
//
//
//

double trend[];
double SilverBuffer[];
double RedBuffer[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{

   IndicatorBuffers(3);
   SetIndexBuffer(0,SilverBuffer);
   SetIndexBuffer(1,RedBuffer);
   SetIndexBuffer(2,trend);
   SetLevelValue(0,0); 
   
   IndicatorShortName("SMAC-D");
return(0);
}

int deinit() 
{  
   if (arrowsVisible) deleteArrows(); 
   
return(0); 
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit=MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //
   
   for (i=limit; i>=0; i--) SilverBuffer[i] = iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_WEIGHTED,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_WEIGHTED,i);
   for (i=limit; i>=0; i--) RedBuffer[i]    = iMAOnArray(SilverBuffer,Bars,SignalSMA,0,MODE_EMA,i);
   for (i=limit; i>=0; i--)
   {
      trend[i] = trend[i+1];
           if (SilverBuffer[i]>RedBuffer[i]) trend[i] = 1;
           if (SilverBuffer[i]<RedBuffer[i]) trend[i] =-1;
           manageArrow(i);
   }
   
   if (alertsOn)
          {
          if (alertsOnCurrent)
               int whichBar = 0;
          else     whichBar = 1;

          //
          //
          //
          //
          //
         
          if (trend[whichBar] != trend[whichBar+1])
          if (trend[whichBar] == 1)
               doAlert("uptrend");
         else  doAlert("downtrend");       
   }
   
   return(0);
}

//+------------------------------------------------------------------+
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," SMAC-D ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," SMAC-D "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}

//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible)
   {
      deleteArrow(Time[i]); 
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsUniqueID+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,arrowsSize);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsUniqueID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsUniqueID+":"+time; ObjectDelete(lookFor);
}

   